@@ -2,7 +2,7 @@
 ;; Copyright (c) 1998 by Shenghuo Zhu <zsh@cs.rochester.edu>
 
 ;; Author: Shenghuo Zhu <zsh@cs.rochester.edu>
-;; $Revision: 5.7 $
+;; $Revision: 5.8 $
 ;; Keywords: news postscript uudecode binhex shar
 
 ;; This file is not part of GNU Emacs, but the same permissions
@@ -139,7 +139,8 @@ This can be either \"inline\" or \"attachment\".")
 			       "application/octet-stream"))
 		     'x-uuencode nil
 		     (if (and file-name (not (equal file-name "")))
-			 (list mm-dissect-disposition (cons 'filename file-name)))))
+			 (list mm-dissect-disposition
+			       (cons 'filename file-name)))))
 	      ((eq type 'binhex)
 	       (mm-make-handle (mm-uu-copy-to-buffer start-char end-char)
 		     (list (or (and file-name
@@ -149,7 +150,8 @@ This can be either \"inline\" or \"attachment\".")
 			       "application/octet-stream"))
 		     'x-binhex nil
 		     (if (and file-name (not (equal file-name "")))
-			 (list mm-dissect-disposition (cons 'filename file-name)))))
+			 (list mm-dissect-disposition
+			       (cons 'filename file-name)))))
 	      ((eq type 'shar)
 	       (mm-make-handle (mm-uu-copy-to-buffer start-char end-char)
 		     '("application/x-shar"))))
